package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.IThirdPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewOutput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.space.service.ISpaceYesterdayGreenRateCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.SpaceOptimizeShortDurationView;
import pps.core.space.entity.SpaceYesterdayGreenRateView;
import pps.core.space.mapper.SpaceOptimizeShortDurationViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 昨日绿电占比Cloud模块
 *
 * @author ZWT
 * @date 2024/11/27 15:17
 */
@XService
public class SpaceYesterdayGreenRateCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceYesterdayGreenRateCloudService {

    /**
     * 昨日绿电占比
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult yesterdayGreenRateJob(XContext context) {
        //取昨日
        DateTime beginOfYesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        //取昨日间开优化井
        SpaceOptimizeShortDurationViewMapper spaceMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        List<SpaceOptimizeShortDurationView> durationViewList = spaceMapper.selectResultList(SpaceOptimizeShortDurationView.builder()
                .optimizeDate(beginOfYesterday)
                .build());
        Map<String, Map<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>>> durationCollect;
        Map<String, Map<Date, BigDecimal>> activeMap;
        Map<String, Map<Date, BigDecimal>> predictionMap;
        if (CollUtil.isEmpty(durationViewList)) {
            durationCollect = Collections.emptyMap();
            activeMap = Collections.emptyMap();
            predictionMap = Collections.emptyMap();
        } else {
            //间开区间Map
            durationCollect = durationViewList.stream()
                    .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getLineName,
                                    Collectors.groupingBy(SpaceOptimizeShortDurationView::getStartSeq,
                                            Collectors.groupingBy(SpaceOptimizeShortDurationView::getIsOptimize))
                            )
                    );
            //取昨日实际发电功率
            IThirdPowerCloudService cloudService = context.getBean(IThirdPowerCloudService.class);
            XListResult<GetThirdActivePowerOutput> result = cloudService.queryPhotovoltaicPower30(context, GetThirdActivePowerInput.builder()
                    .stationNames(durationCollect.keySet())
                    .createDate(beginOfYesterday)
                    .build());
            result.throwIfFail();
            List<GetThirdActivePowerOutput> powerList = result.getResult();
            if (CollUtil.isEmpty(powerList)) {
                activeMap = Collections.emptyMap();
            } else {
                activeMap = powerList.stream()
                        .collect(Collectors.groupingBy(GetThirdActivePowerOutput::getStationName,
                                Collectors.toMap(GetThirdActivePowerOutput::getCreateDate, GetThirdActivePowerOutput::getPhotovoltaicPower)));
            }
            //昨日预测发电功率
            IPlantPredictedPowerCloudService service = context.getBean(IPlantPredictedPowerCloudService.class);
            XListResult<GetPredictedPowerViewOutput> predictionResult = service.queryPredictionPower30(context, GetPredictedPowerViewInput.builder()
                    .stationNames(durationCollect.keySet())
                    .dataDate(beginOfYesterday)
                    .build());
            predictionResult.throwIfFail();
            List<GetPredictedPowerViewOutput> predictionList = predictionResult.getResult();
            if (CollUtil.isEmpty(predictionList)) {
                predictionMap = Collections.emptyMap();
            } else {
                predictionMap = predictionList.stream()
                        .collect(Collectors.groupingBy(GetPredictedPowerViewOutput::getStationName,
                                Collectors.toMap(GetPredictedPowerViewOutput::getCreateTime, GetPredictedPowerViewOutput::getPredictPower)));
            }
        }
        //初始化时间轴
        List<DateTime> rangeToList = DateUtil.rangeToList(DateUtil.offsetDay(BusinessConstant.DATE_FLAG, -1), BusinessConstant.DATE_FLAG, DateField.MINUTE, 30);
        String lineName;
        Map<Date, BigDecimal> activePowerMap;
        Map<Date, BigDecimal> predictionPowerMap;
        BigDecimal wellGreenPowerActual;
        BigDecimal wellGreenPowerOptimize;
        BigDecimal runHourActual;
        BigDecimal runHourOptimize;
        List<SpaceOptimizeShortDurationView> unOptimizeList;
        List<SpaceOptimizeShortDurationView> optimizeList;
        BigDecimal activePower;
        BigDecimal predictionPower;
        TreeMap<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>> wellSortMap;
        SpaceOptimizeShortDurationView durationTemp;
        BigDecimal serviceRating;
        BigDecimal sumServiceRating;
        List<SpaceYesterdayGreenRateView> batchList = new ArrayList<>(durationCollect.size() * 2);
        SpaceOptimizeShortDurationView unOptimizeDuration;
        SpaceOptimizeShortDurationView optimizeDuration;
        DateTime openWellTime;
        DateTime closeWellTime;
        BigDecimal wellTotalPowerActual;
        BigDecimal wellTotalPowerOptimize;
        for (Map.Entry<String, Map<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>>> lineEntry : durationCollect.entrySet()) {
            lineName = lineEntry.getKey();
            activePowerMap = activeMap.getOrDefault(lineName, Collections.emptyMap());
            predictionPowerMap = predictionMap.getOrDefault(lineName, Collections.emptyMap());
            //按顺序排
            wellSortMap = new TreeMap<>(lineEntry.getValue());
            sumServiceRating = BigDecimal.ZERO;
            //计算结果
            for (Map.Entry<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>> sortEntry : wellSortMap.entrySet()) {
                //未优化间开
                unOptimizeList = sortEntry.getValue().get(1);
                if (CollUtil.isEmpty(unOptimizeList)) {
                    continue;
                }
                wellGreenPowerActual = BigDecimal.ZERO;
                wellGreenPowerOptimize = BigDecimal.ZERO;
                runHourActual = BigDecimal.ZERO;
                runHourOptimize = BigDecimal.ZERO;
                durationTemp = unOptimizeList.get(0);
                //累加运行功率
                serviceRating = ObjectUtil.defaultIfNull(durationTemp.getServiceRating(), BigDecimal.ZERO);
                sumServiceRating = sumServiceRating.add(serviceRating);
                //优化后间开
                optimizeList = sortEntry.getValue().get(0);
                boolean optimizeListFlag = CollUtil.isNotEmpty(optimizeList);
                int unOptimizeIndex = 0;
                int optimizeIndex = 0;
                //计算优化前后功率
                for (DateTime dateTime : rangeToList) {
                    //计算实际
                    activePower = activePowerMap.getOrDefault(dateTime, BigDecimal.ZERO);
                    unOptimizeDuration = unOptimizeList.get(unOptimizeIndex);
                    //时间转换
                    openWellTime = this.parseDateTime(unOptimizeDuration.getOpenWellTime());
                    closeWellTime = this.parseDateTime(unOptimizeDuration.getCloseWellTime());
                    if (DateUtil.compare(dateTime, openWellTime) >= 0 && DateUtil.compare(dateTime, closeWellTime) < 0) {
                        //计算发电功率
                        if (activePower.compareTo(BigDecimal.ZERO) == 0) {
                            //不管
                        } else if (activePower.compareTo(sumServiceRating) >= 0) {
                            //光伏发电功率大于运行负荷: 该时段光伏用电量量=运行负荷*0.5
                            wellGreenPowerActual = wellGreenPowerActual.add(serviceRating.multiply(BusinessConstant.BIG_DECIMAL_0_5));
                        } else {
                            //光伏发电功率小于井的运行负荷: 该时段光伏用电量=光伏发电功率*0.5
                            wellGreenPowerActual = wellGreenPowerActual.add(activePower.multiply(BusinessConstant.BIG_DECIMAL_0_5));
                        }
                        //累加开井时长
                        runHourActual = runHourActual.add(BusinessConstant.BIG_DECIMAL_0_5);
                    } else if (DateUtil.compare(dateTime, closeWellTime) >= 0) {
                        unOptimizeIndex++;
                    }
                    if (unOptimizeIndex == unOptimizeList.size()) {
                        unOptimizeIndex = unOptimizeList.size() - 1;
                    }
                    /*--------------------------------------------------*/
                    if (optimizeListFlag) {
                        predictionPower = predictionPowerMap.getOrDefault(dateTime, BigDecimal.ZERO);
                        optimizeDuration = optimizeList.get(optimizeIndex);
                        //时间转换
                        openWellTime = this.parseDateTime(optimizeDuration.getOpenWellTime());
                        closeWellTime = this.parseDateTime(optimizeDuration.getCloseWellTime());
                        if (DateUtil.compare(dateTime, openWellTime) >= 0 && DateUtil.compare(dateTime, closeWellTime) < 0) {
                            //计算发电功率
                            if (predictionPower.compareTo(BigDecimal.ZERO) == 0) {
                                //不管
                            } else if (predictionPower.compareTo(sumServiceRating) >= 0) {
                                //光伏发电功率大于运行负荷: 该时段光伏用电量量=运行负荷*0.5
                                wellGreenPowerOptimize = wellGreenPowerOptimize.add(serviceRating.multiply(BusinessConstant.BIG_DECIMAL_0_5));
                            } else {
                                //光伏发电功率小于井的运行负荷: 该时段光伏用电量=光伏发电功率*0.5
                                wellGreenPowerOptimize = wellGreenPowerOptimize.add(predictionPower.multiply(BusinessConstant.BIG_DECIMAL_0_5));
                            }
                            //累加开井时长
                            runHourOptimize = runHourOptimize.add(BusinessConstant.BIG_DECIMAL_0_5);
                        } else if (DateUtil.compare(dateTime, closeWellTime) >= 0) {
                            optimizeIndex++;
                        }
                        if (optimizeIndex == optimizeList.size()) {
                            optimizeIndex = optimizeList.size() - 1;
                        }
                    }
                }
                //添加计算结果
                wellTotalPowerActual = runHourActual.multiply(serviceRating);
                wellTotalPowerOptimize = runHourOptimize.multiply(serviceRating);
                SpaceYesterdayGreenRateView yesterdayGreenRateView = new SpaceYesterdayGreenRateView();
                BaseUtils.setBaseModelDefaultForJob(yesterdayGreenRateView);
                yesterdayGreenRateView.setRecordDate(beginOfYesterday);
                yesterdayGreenRateView.setWellheadId(durationTemp.getWellheadId());
                yesterdayGreenRateView.setWellNumber(durationTemp.getWellNumber());
                yesterdayGreenRateView.setStartSeq(sortEntry.getKey());
                yesterdayGreenRateView.setWellTotalPowerActual(wellTotalPowerActual);
                yesterdayGreenRateView.setWellTotalPowerOptimize(wellTotalPowerOptimize);
                yesterdayGreenRateView.setWellGreenPowerActual(wellGreenPowerActual);
                yesterdayGreenRateView.setWellGreenPowerOptimize(wellGreenPowerOptimize);
                yesterdayGreenRateView.setGreenPowerRateActual(BaseUtils.getRate(wellGreenPowerActual, wellTotalPowerActual));
                yesterdayGreenRateView.setGreenPowerRateOptimize(BaseUtils.getRate(wellGreenPowerOptimize, wellTotalPowerOptimize));
                batchList.add(yesterdayGreenRateView);
            }
        }
        return XServiceResult.OK;
    }

    /**
     * 转换时间
     *
     * @param timeString 时间字符串
     * @return {@link DateTime }
     */
    private DateTime parseDateTime(String timeString) {
        DateTime wellTime;
        if (CharSequenceUtil.equals(timeString, BusinessConstant.END_OF_DAY_TIME)) {
            wellTime = BusinessConstant.DATE_FLAG;
        } else {
            wellTime = DateUtil.parse(timeString + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
        }
        return wellTime;
    }
}